/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <cstdlib>
#include <string>
#include "audio.h"

namespace Tuner {
	const AudioInfo DefValues = {
		DUPLEX,
		-1,
		-1,
		U8,
		MONO,
		22050
	};

	const char *AudioKeyVal[6] = { "AudioMode", "FragNumber", "FragSize", "Format",
		"Channels", "SamplingRate" };
}

bool
Tuner::AudioInfo::load(Config::ConfFile &cf) throw(Error::AudioError)
{
	std::string val[6];
	int iVal[6];
	for(int i = 0; i < 6; ++i)
		cf.getValue(AudioKeyVal[i], val[i]), iVal[i] = std::atoi(val[i].c_str());

	if(val[0] != "" && iVal[0] >= PLAYBACK && iVal[0] <= DUPLEX)
		mode = OpenMode(iVal[0]);

	if(val[1] != "")
		fragNumber = iVal[1];
	if(val[2] != "")
		fragSize = iVal[2];

	if(val[3] != "" && iVal[3] >= U8 && iVal[3] <= S16LE)
		fmt = Format(iVal[3]);

	if(val[4] != "" && iVal[4] >= MONO && iVal[4] <= STEREO)
		chn = Channels(iVal[4]);

	if(val[5] != "")
		rate = iVal[5];

	return true;
}

bool
Tuner::AudioInfo::loadDefaults() throw()
{
	*this = DefValues;
	return true;
}

bool
Tuner::AudioInfo::save(Config::ConfFile &cf) throw(Error::AudioError)
{
	char buf[6][25];
	std::sprintf(buf[0], "%d", mode);
	std::sprintf(buf[1], "%d", fragNumber);
	std::sprintf(buf[2], "%d", fragSize);
	std::sprintf(buf[3], "%d", fmt);
	std::sprintf(buf[4], "%d", chn);
	std::sprintf(buf[5], "%d", rate);

	for(int i = 0; i < 6; ++i)
		cf.setValue(AudioKeyVal[i], buf[i]);

	return true;
}

