/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOMEGUI_H_
#define _GNOMEGUI_H_

#include <queue>
#include <gnome.h>
#include "gnomebase.h"
#include "gui.h"
#include "posixthread.h"
#include "note.h"

namespace Interface {
	class GnomeGui: public Gui, private GnomeBase {
		public:
			GnomeGui(int *, char ***) throw();
			~GnomeGui();

		void open() throw(Error::GuiError);
		void close() throw(Error::GuiError);

		void start() throw(Error::GuiError);
		void stop() throw(Error::GuiError);

		void pause() throw(Error::GuiError);
		void resume() throw(Error::GuiError);

		void delay(unsigned int) throw(Error::GuiError);

		void display(const Tuner::Note &, bool = true) throw(Error::GuiError);
		void display(const std::string &) throw(Error::GuiError);

		void errorMsg(const std::string &) throw();

		void refresh() throw();
		void expose(GdkEventExpose *) throw();

		void freeze() throw();
		void thaw() throw();

		private:
			GnomeGui(const GnomeGui &);
		const GnomeGui &operator=(const GnomeGui &);

		void _display() throw();
		void _display(const std::string &, DialogType) throw();
		void _messages() throw();

		void initMenu() throw();
		void initDisplay() throw();
		void addHandlers() throw();
		void loadFiles() throw();

		void destroy() throw();

		bool isOpen, paused;

		Tuner::Note curr;
		Thread::Resource currLock, queueLock;
		int tag;
		enum { FREEZE, THAW, NONE } request;

		struct DInfo {
			std::string message;
			DialogType dt;
		};

		std::queue<DInfo> msgQueue;

		GtkWidget *mainWindow;
		GtkWidget *vBox;
		GtkWidget *drawingArea;

		struct Line {
			int x, y;
			int x1, y1;
		};

		Line ends[101];
		GdkFont *font;
		GdkPixmap *pix, *back, *file[5];
		int width, height;
	};

	inline void GnomeGui::refresh() throw()
	{
		if(!paused)
			_display();
		_messages();
	}
}

#endif

