/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _MAIN_THREAD_H_
#define _MAIN_THREAD_H_

#include <new>
#include <stdexcept>
#include "audio.h"
#include "objfactory.h"
#include "posixthread.h"
#include "utils.h"

namespace Config {
	class GenConf;
}

namespace Interface {
	class Gui;
}

namespace Tuner {
	class Calc;
}

namespace Thread {
	class MainThread: public PosixThread {
		public:
			MainThread(Interface::Gui *, Config::GenConf &, int)
				throw(Error::AudioError, Error::CalcError, Error::ThreadError,
				std::bad_alloc);
		   ~MainThread();

		void start() throw(Error::ThreadError);
		void stop() throw(Error::ThreadError);
		void pause() throw(Error::ThreadError);
		void resume() throw(Error::ThreadError);
		void reconfigure(int) throw();

		private:
		enum Command { STOP, PAUSE, CONTINUE, RECONFIGURE };

			MainThread(const MainThread &);
		const MainThread &operator=(const MainThread &);

		enum Response { DONE };

		int main() throw();

		void adjustBuffers(unsigned int) throw();
		void openDevice(bool) throw();
		void closeDevice() throw();
		void setup() throw(std::bad_alloc);

		Tuner::ObjFactory fact;

		Interface::Gui *gui;
		Tuner::Audio *&audioDev;
		Tuner::Calc *&calc;
		Tuner::Buffer<unsigned char> *&buffer;
		Tuner::AudioInfo ai;

		Tuner::Buffer<unsigned char> auxBuf;
		int refresh;
		unsigned int refreshBytes;
		bool paused;
	};

	inline Thread::MainThread::~MainThread() { }
}

#endif

