/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _PLAYERTHREAD_H_
#define _PLAYERTHREAD_H_

#include <vector>
#include "audio.h"
#include "objfactory.h"
#include "playererror.h"
#include "posixthread.h"
#include "preset.h"
#include "utils.h"

namespace Config {
	class GenConf;
}

namespace Player {
	using Thread::Lock;
	using Thread::PosixThread;
	using Thread::Resource;
	using Tuner::Audio;
	using Tuner::AudioInfo;
	using Tuner::Buffer;
	using Tuner::ObjFactory;

	class PlayerThread: private PosixThread {
		public:
			PlayerThread(Config::GenConf &) throw(Error::AudioError,
				Error::ThreadError, std::bad_alloc);
			~PlayerThread();

		const Preset &select(int) throw();
		const std::vector<Preset> &listPresets() const throw();
		void play(int) throw(Error::PlayerError, Error::ThreadError);

		void start() throw(Error::AudioError, Error::PlayerError, Error::ThreadError);
		void stop() throw(Error::ThreadError);

		private:
		enum Command { PLAY, STOP };

			PlayerThread(const PlayerThread &);
		const PlayerThread &operator=(const PlayerThread &);

		void loadPresets() throw();
		int main() throw();

		ObjFactory fact;

		Resource presLock;
		int selected;
		std::vector<Preset> presets;

		Audio *&audioDev;
		AudioInfo devInfo;

		Buffer<unsigned char> buf;

		bool playing;
	};

	inline const Preset &PlayerThread::select(int i) throw()
	{
		Lock(Resource(presLock));
		return presets[selected = i];
	}

	inline const std::vector<Preset> &PlayerThread::listPresets() const throw()
	{
		return presets;
	}

}

#endif

