/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cmath>
#include <cstdlib>
#include "window.h"

void
Tuner::Window::init(int n, WindowType t) throw(std::bad_alloc)
{
	if(size != n) {
		if(size != -1)
			destroy();
		winData = new double[size = n];
	} else {
		if(type == t)
			return;
	}

	int i, j, mid = size / 2, mm = (size - 1) / 2, mp = (size + 1) / 2;
	switch(type = t) {
		case NONE:
		case RECTANGULAR:
			for(i = 0, j = size - 1; i <= mid; ++i, --j)
				winData[i] = winData[j] = 1.0;
			break;
		case HANNING:
			for(i = 0, j = size - 1; i <= mid; ++i, --j)
				winData[i] = winData[j] = 0.5 - 0.5 *
					std::cos(2*M_PI * i / size);
			break;
		case PARZEN:
			for(i = 0, j = size - 1; i <= mid; ++i, --j)
				winData[i] = winData[j] = 1.0 -
					std::abs((i - mm)/mp);
			break;
		case BARTLETT:
			for(i = 0, j = size - 1; i <= mid; ++i, --j)
				winData[i] = winData[j] = i * (1.0 / mid);
			break;
		case HAMMING:
			for(i = 0, j = size - 1; i <= mid; ++i, --j)
				winData[i] = winData[j] = 0.54 - 0.46 *
					std::cos(2*M_PI * i / size);
			break;
		case BLACKMAN:
			for(i = 0, j = size - 1; i <= mid; ++i, --j) {
				double cosx = std::cos(2*M_PI * i / size);
				winData[i] = winData[j] = 0.34401 + cosx *
					(cosx * 0.15844 - 0.49755);
			}
			break;
	}
}

